!WRF:DRIVER_LAYER:INTEGRATION
!

MODULE module_spectralnudging_driver


CONTAINS

!------------------------------------------------------------------
   SUBROUTINE spectralnudging_driver(itimestep,dt,xtime,                   &
                  id,  &
                  RUNDGDTEN,RVNDGDTEN,RTHNDGDTEN,                 &
                  RQVNDGDTEN,RMUNDGDTEN,                          &
                  u_ndg_old,v_ndg_old,t_ndg_old,q_ndg_old,mu_ndg_old,       &
                  u_ndg_new,v_ndg_new,t_ndg_new,q_ndg_new,mu_ndg_new,       &
                  sig_a,                                                    &
                  u3d,v3d,th_phy,rho,moist,                       &
                  p_phy,pi_phy,p8w,t_phy,dz8w,z,z_at_w,           &
                  config_flags,DX,n_moist,                        &
                  STEPFG,                                         &
                  pblh,ht,                                        &
                  ids,ide, jds,jde, kds,kde,                      &
                  ims,ime, jms,jme, kms,kme,                      &
                  i_start,i_end, j_start,j_end, kts,kte, num_tiles)
!------------------------------------------------------------------
   USE module_configure
   USE module_state_description
   USE module_model_constants

   TYPE(grid_config_rec_type),  INTENT(IN   )    :: config_flags
!

   INTEGER , INTENT(IN)         ::     id

   INTEGER,    INTENT(IN   )    ::     ids,ide, jds,jde, kds,kde, &
                                       ims,ime, jms,jme, kms,kme, &
                                       kts,kte, num_tiles,        &
                                       n_moist

   INTEGER, DIMENSION(num_tiles), INTENT(IN) ::                   &
  &                                    i_start,i_end,j_start,j_end

   INTEGER,    INTENT(IN   )    ::     itimestep,STEPFG
!
   REAL,       INTENT(IN   )    ::     DT,DX,XTIME


!
   REAL,       DIMENSION( ims:ime, kms:kme, jms:jme ),            &
               INTENT(INOUT )    ::                         p_phy, &
                                                          pi_phy, &
                                                             p8w, &
                                                             rho, &
                                                           t_phy, &
                                                             u3d, &
                                                             v3d, &
                                                            dz8w, &
                                                               z, &
                                                          z_at_w, &
                                                          th_phy
!
   REAL, DIMENSION( ims:ime, kms:kme, jms:jme, n_moist ),         &
         INTENT(IN ) ::                                    moist
!
!
!
   REAL,       DIMENSION( ims:ime, kms:kme, jms:jme ),            &
               INTENT(INOUT)    ::                       RUNDGDTEN, &
                                                         RVNDGDTEN, &
                                                        RTHNDGDTEN, &
                                                        RQVNDGDTEN

   REAL,       DIMENSION( ims:ime,  jms:jme ),            &
               INTENT(INOUT)    ::                      RMUNDGDTEN

   REAL,       DIMENSION( ims:ime, kms:kme, jms:jme ),            &
               INTENT(INOUT)    ::                       u_ndg_old, &
                                                         v_ndg_old, &
                                                         t_ndg_old, &
                                                         q_ndg_old, &
                                                         u_ndg_new, &
                                                         v_ndg_new, &
                                                         t_ndg_new, &
                                                         q_ndg_new
   REAL,       DIMENSION( ims:ime,  jms:jme ),            &
               INTENT(INOUT)    ::                       mu_ndg_old, &
                                                         mu_ndg_new

!
   REAL,    DIMENSION( ims:ime , jms:jme ),     &
               INTENT(IN   ) ::           pblh, &
                                            ht

!  LOCAL  VAR

!
   INTEGER :: i,J,K,NK,jj,ij,kk

   real, dimension(kme-kms) :: sig_a              !half level eta value

   INTEGER   ::      analysis_interval, nudging_interval

   !  Local data.

   analysis_interval = config_flags%gfdda_interval_m   !minute

   xtime_old = FLOOR(xtime/analysis_interval) * analysis_interval * 1.0   ! minute
   xtime_new = xtime_old + analysis_interval                              ! minute

   nudging_interval = 3     ! nudging interval is 3 minute.

   if (mod(int(xtime),nudging_interval) .eq. 0) then

         !spectral nudging u
         call spectral(u3d(ids:ide-1,kds:kde-1,jds:jde-1),            &
                       u_ndg_new(ids:ide-1,kds:kde-1,jds:jde-1),      &
                       u_ndg_old(ids:ide-1,kds:kde-1,jds:jde-1),      &
                       xtime,xtime_new,xtime_old,                   &
                       ids,ide-1,jds,jde-1,kds,kde-1,sig_a)
         !spectral nudging v
         call spectral(v3d(ids:ide-1,kds:kde-1,jds:jde-1),            &
                       v_ndg_new(ids:ide-1,kds:kde-1,jds:jde-1),      &
                       v_ndg_old(ids:ide-1,kds:kde-1,jds:jde-1),      &
                       xtime,xtime_new,xtime_old,                   &
                       ids,ide-1,jds,jde-1,kds,kde-1,sig_a)
   endif

   end subroutine spectralnudging_driver


SUBROUTINE spectral(fval,avala,avalb,xtime,xtime_new,xtime_old,ids,ide,jds,jde,kds,kde,sig_a)
!     zvfk : intensification factor for vertical nesting
!     fval : forecast variables at next time step
!     aval : large-scale variables at the correspponding time
!     avala: 6 hourly interval large-scale variables at later   time step
!     avalb: 6 hourly interval large-scale variables at earlier time step
!     index: variable index to identify cross or dot point value

      real, parameter :: zvfk = 0.1
      real      :: xtime, xtime_new, xtime_old
      integer   :: ids,ide,jds,jde,kds,kde

      real, dimension(ide-ids+1,kde-kds+1,jde-jds+1) :: fval
      real, dimension(ide-ids+1,kde-kds+1,jde-jds+1) :: aval
      real, dimension(ide-ids+1,kde-kds+1,jde-jds+1) :: avala, avalb

      real, dimension(kde-kds+1) :: sig_a              !half level eta value

!     fspec, aspec: internal storage fields for spectral nudging

      real, dimension(ide-ids+1,jde-jds+1) :: fspec,   aspec

!     spectral coefficients for in- and output fields

      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aa_s, ab_s, ba_s, bb_s
      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aar_s, abr_s, bar_s, bbr_s
      real, dimension((ide-ids)/2+1,(jde-jds)/2+1) :: aa_n, ab_n, ba_n, bb_n

!     dimensions for local fields
!     zwgtwd       : vertical weighting profile
!     ftmp1, ftmp2 : temporary local array for forecast value
!     atmp1, atmp2 : temporary local array for analysis value

      real,dimension(kde-kds+1) :: zwgtvd
      real,dimension(ide-ids+1,jde-jds+1) :: ftmp1, atmp1
      real,dimension(ide-ids+1,jde-jds+1) :: ftmp2, atmp2

!c.... preparations
!$omp parallel do &
!$omp private(ii,jj)
      do 10 ii = 1, ((ide-ids)/2+1)
      do 10 jj = 1, ((jde-jds)/2+1)
         aa_n(ii,jj) = 0.
         ab_n(ii,jj) = 0.
         ba_n(ii,jj) = 0.
         bb_n(ii,jj) = 0.
   10 continue
!$omp end parallel do


!.... Linear interpolation to determine large-scale values at corresponding time

      xm = xtime  - xtime_old
      xn = xtime_new - xtime
      !print*,"xm=",xm,"xn=",xn
!$omp parallel do &
!$omp private(kk,ii,jj)
      do 20 kk = 1,kde-kds+1
      do 20 jj = 1,jde-jds+1
      do 20 ii = 1,ide-ids+1
      aval(ii,kk,jj) = (xm*avala(ii,kk,jj)+xn*avalb(ii,kk,jj))/(xm+xn)
   20 continue
!$omp end parallel do
!.... assign of required local fields

      do 31 k = 1,kde-kds+1
         zwgtvd(k) = 0.5*zvfk*(1.0-min(sig_a(k),1.0)/1.0)**2
   31 continue

!.... preparation of spectral decomposition: calculation of (is,js) from (i,j)

      isc = 7   !lon
      jsc = 6   !lat

      is  = (ide-ids)/2 + 1
      js  = (jde-jds)/2 + 1
      do 100 k = 1,kde-kds+1
!$omp parallel do &
!$omp private(ii,jj)
      do 40 jj = 1,jde-jds+1
      do 40 ii = 1,ide-ids+1
         fspec(ii,jj) = fval(ii,k,jj)
         aspec(ii,jj) = aval(ii,k,jj)
   40 continue
!$omp end parallel do
      call speczer(ide-ids+1,jde-jds+1,is,js,isc,jsc,fspec,aa_s ,ab_s, ba_s ,bb_s)
      call speczer(ide-ids+1,jde-jds+1,is,js,isc,jsc,aspec,aar_s,abr_s,bar_s,bbr_s)

!$omp parallel do &
!$omp private(ii,jj)
      do 70 jj = 1,jsc
      do 70 ii = 1,isc 
         aa_n(ii,jj) = aa_s(ii,jj) - zwgtvd(k)*(aa_s(ii,jj)-aar_s(ii,jj))
         ab_n(ii,jj) = ab_s(ii,jj) - zwgtvd(k)*(ab_s(ii,jj)-abr_s(ii,jj))
         ba_n(ii,jj) = ba_s(ii,jj) - zwgtvd(k)*(ba_s(ii,jj)-bar_s(ii,jj))
         bb_n(ii,jj) = bb_s(ii,jj) - zwgtvd(k)*(bb_s(ii,jj)-bbr_s(ii,jj))
   70 continue
!$omp end parallel do

!c.... re-composition of spectral coefficients to fields
      call speczus(ide-ids+1,jde-jds+1,is,js,isc,jsc,                   &
           fspec,aa_s,ab_s,ba_s,bb_s,aa_n,ab_n,ba_n,bb_n)

!$omp parallel do &
!$omp private(ii,jj)
      do 90 jj = 1,jde-jds+1
      do 90 ii = 1,ide-ids+1
        fval(ii,k,jj) = fspec(ii,jj)
   90 continue
!$omp end parallel do
  
  100 continue

      return
      end subroutine spectral

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      subroutine speczer                                          &
                (ii,jj,is,js,isc,jsc,feld,aafin,abfin,bafin,bbfin)

      dimension feld(ii,jj)  
      dimension asem(is,jj)  , bsem(is,jj)
      dimension aafin(is,js), abfin(is,js)
      dimension bafin(is,js), bbfin(is,js)


      twopi = 8.*atan(1.)
!$omp parallel do &
!$omp private(ms,ns)
      do 10 ns = 1,js
      do 10 ms = 1,is
         aafin(ms,ns) = 0.
         abfin(ms,ns) = 0.
         bafin(ms,ns) = 0.
         bbfin(ms,ns) = 0.
   10 continue
!$omp end parallel do

!$omp parallel do &
!$omp private(ms,n)
      do 20 n  = 1,jj
      do 20 ms = 1,is
         asem(ms,n) = 0.
                                                      bsem(ms,n) = 0.
   20 continue
!$omp end parallel do

!.... In x-direction

!$omp parallel do &
!$omp private(ms,n)
      do 30 n  = 1,jj
      do 30 ms = 1,isc
      do 30 ir = 1,ii
         asem(ms,n) = asem(ms,n)+(feld(ir,n)*cos(twopi*real(ms-1)*      &
                     real(ir)/real(ii)))/real(ii)
         bsem(ms,n) = bsem(ms,n)+(feld(ir,n)*sin(twopi*real(ms-1)*      &
                     real(ir)/real(ii)))/real(ii)
   30 continue

!.... In Y-direction

!$omp parallel do &
!$omp private(ms,ns)
      do 40 ms = 1,isc
      do 40 ns = 1,jsc
      do 40 jr = 1,jj
         aafin(ms,ns) = aafin(ms,ns)+(asem(ms,jr)*cos(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         bafin(ms,ns) = bafin(ms,ns)+(asem(ms,jr)*sin(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         abfin(ms,ns) = abfin(ms,ns)+(bsem(ms,jr)*cos(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
         bbfin(ms,ns) = bbfin(ms,ns)+(bsem(ms,jr)*sin(twopi*real(ns-1)* &
                       real(jr)/real(jj)))/real(jj)
   40 continue
      return
      end subroutine speczer
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1

      subroutine speczus(ii,jj,is,js,isc,jsc,feld,              &
                        aafin,abfin,bafin,bbfin,                &
                        aafin_n,abfin_n,bafin_n,bbfin_n)

      dimension feld(ii,jj),    asem(is,jj),   bsem(is,jj)
      dimension suma(is,jj),   sumb(is,jj),   sum(ii,jj)
      dimension aafin(is,js), bafin(is,js), abfin(is,js), bbfin(is,js)
      dimension asem_n(is,jj), bsem_n(is,jj)
      dimension suma_n(is,jj), sumb_n(is,jj), sum_n(ii,jj)

      dimension aafin_n(is,js), abfin_n(is,js)
      dimension bafin_n(is,js), bbfin_n(is,js)

      twopi = 8.*atan(1.)
!$omp parallel do &
!$omp private(m,n)
      do 10 n = 1,jj
      do 10 m = 1,ii
         sum(m,n)   = 0.
         sum_n(m,n) = 0.
   10 continue
!$omp end parallel do
!$omp parallel do &
!$omp private(ms,n)
      do 20 n  = 1,jj
      do 20 ms = 1,is
         suma  (ms,n) = 0.
         sumb  (ms,n) = 0.
         asem  (ms,n) = 0.
         bsem  (ms,n) = 0.
         suma_n(ms,n) = 0.
         sumb_n(ms,n) = 0.
         asem_n(ms,n) = 0.
         bsem_n(ms,n) = 0.
   20 continue
!$omp end parallel do

!.....Spectral recomposition: fist in j dimension

!$omp parallel do &
!$omp private(ms,n)
      do 30 n  = 1,jj
      do 30 ms = 1,isc
      do 25 ns = 2,jsc
       suma(ms,n)   = suma(ms,n)   + aafin(ms,ns)*cos(twopi*real(ns-1)  &
                                  * real(n)/real(jj))                   &
                                  + bafin(ms,ns)*sin(twopi*real(ns-1)   &
                                  * real(n)/real(jj))
       sumb(ms,n)   = sumb(ms,n)   + abfin(ms,ns)*cos(twopi*real(ns-1)  &
                                  * real(n)/real(jj))                   &
                                  + bbfin(ms,ns)*sin(twopi*real(ns-1)   &
                                  * real(n)/real(jj))
       suma_n(ms,n) = suma_n(ms,n) + aafin_n(ms,ns)*cos(twopi*real(ns-1) &
                                 * real(n)/real(jj))                    &
                                  + bafin_n(ms,ns)*sin(twopi*real(ns-1)  &
                                  * real(n)/real(jj))
       sumb_n(ms,n) = sumb_n(ms,n) + abfin_n(ms,ns)*cos(twopi*real(ns-1) &
                                  * real(n)/real(jj))                    &
                                  + bbfin_n(ms,ns)*sin(twopi*real(ns-1)  &
                                  * real(n)/real(jj))
   25 continue
      asem  (ms,n) = aafin  (ms,1) + 2.*suma  (ms,n)
      bsem  (ms,n) = abfin  (ms,1) + 2.*sumb  (ms,n)
      asem_n(ms,n) = aafin_n(ms,1) + 2.*suma_n(ms,n)
      bsem_n(ms,n) = abfin_n(ms,1) + 2.*sumb_n(ms,n)
   30 continue
!$omp end parallel do

!.....Spectral recomposition: fist in i dimension

!chacha
!$omp parallel do &
!$omp private(m,n)
!chacha
      do 40 n  = 1,jj
      do 40 m  = 1,ii
      do 35 ms = 2,isc
         sum(m,n) = sum(m,n)     + asem(ms,n)*cos(twopi*real(ms-1)    &
                                * real(m)/real(ii))                   &
                                + bsem(ms,n)*sin(twopi*real(ms-1)     &
                                * real(m)/real(ii))
         sum_n(m,n) = sum_n(m,n) + asem_n(ms,n)*cos(twopi*real(ms-1)  &
                                * real(m)/real(ii))                   &
                                + bsem_n(ms,n)*sin(twopi*real(ms-1)   &
                                * real(m)/real(ii))
   35 continue
      feld(m,n) = feld(m,n) - asem  (1,n) - 2.*sum  (m,n)   &
                           + asem_n(1,n) + 2.*sum_n(m,n)
   40 continue
!$omp end parallel do
      return
      end subroutine speczus

END MODULE module_spectralnudging_driver
