#!/bin/csh
setenv OMP_NUM_THREADS 8
set defdir = /home/atmos77/wrf/WRFV2.2_sp98
set metdir = /disk1/dklee/atmos77/wrfrcm/metgrid_out
set realdir = /disk1/dklee/atmos77/wrfrcm/real_out
cp $defdir/run/real.exe $metdir

cd $metdir

cat > namelist.input << EOF
 &time_control
 run_days                            = 0,
 run_hours                           = 0,
 run_minutes                         = 0,
 run_seconds                         = 0,
 start_year                          = 1994, 2000, 2000,
 start_month                         = 05,   01,   01,
 start_day                           = 01,   24,   24,
 start_hour                          = 00,   12,   12,
 start_minute                        = 00,   00,   00,
 start_second                        = 00,   00,   00,
 end_year                            = 1994, 2000, 2000,
 end_month                           = 06,   01,   01,
 end_day                             = 01,   25,   25,
 end_hour                            = 00,   12,   12,
 end_minute                          = 00,   00,   00,
 end_second                          = 00,   00,   00,
 interval_seconds                    = 21600
 input_from_file                     = .true.,.false.,.false.,
 history_interval                    = 180,  60,   60,
 frames_per_outfile                  = 248, 1000, 1000,
 restart                             = .false.,
 restart_interval                    = 44640,
 io_form_history                     = 2
 io_form_restart                     = 2
 io_form_input                       = 2
 io_form_boundary                    = 2
 debug_level                         = 0
 auxinput1_inname                    = "met_em.d<domain>.<date>"
 /
 auxinput1_inname                    = "wrf_real_input_em.d<domain>.<date>"

 &domains
 time_step                           = 90,
 time_step_fract_num                 = 0,
 time_step_fract_den                 = 1,
 max_dom                             = 1,
 s_we                                = 1,     1,     1,
 e_we                                = 240,    112,   94,
 s_sn                                = 1,     1,     1,
 e_sn                                = 200,    97,    91,
 s_vert                              = 1,     1,     1,
 e_vert                              = 28,    28,    28,
 num_metgrid_levels                  = 18,
 dx                                  = 30000, 10000,  3333,
 dy                                  = 30000, 10000,  3333,
 grid_id                             = 1,     2,     3,
 parent_id                           = 0,     1,     2,
 i_parent_start                      = 0,     31,    30,
 j_parent_start                      = 0,     17,    30,
 parent_grid_ratio                   = 1,     3,     3,
 parent_time_step_ratio              = 1,     3,     3,
 feedback                            = 1,
 smooth_option                       = 0,
 p_top_requested                     = 5000,
 interp_type                         = 1,
 lowest_lev_from_sfc                 = .false.,
 lagrange_order                      = 1,
 force_sfc_in_vinterp                = 1,
 zap_close_levels                    = 500,
 sfcp_to_sfcp                        = .false.,
 adjust_heights                      = .false.,
 eta_levels                          = 1.000, 0.990, 0.978, 0.964, 0.946, 
                                       0.922, 0.894, 0.860, 0.817, 0.766, 
                                       0.707, 0.644, 0.576, 0.507, 0.444, 
                                       0.380, 0.324, 0.273, 0.228, 0.188, 
                                       0.152, 0.121, 0.093, 0.069, 0.048, 
                                       0.029, 0.014, 0.000, 
 /
 eta_levels                          = 1.000, 0.993, 0.983, 0.970, 0.954,
                                       0.934, 0.909, 0.880, 0.845, 0.807,
                                       0.765, 0.719, 0.672, 0.622, 0.571,
                                       0.520, 0.468, 0.420, 0.376, 0.335,
                                       0.298, 0.263, 0.231, 0.202, 0.175,
                                       0.150, 0.127, 0.106, 0.088, 0.070,
                                       0.055, 0.040, 0.026, 0.013, 0.000

 &physics
 mp_physics                          = 3,     3,     3,
 ra_lw_physics                       = 3,     1,     1,
 ra_sw_physics                       = 3,     1,     1,
 radt                                = 30,    30,    30,
 cam_abs_freq_s                      = 21600,
 levsiz                              = 59,
 paerlev                             = 29,
 cam_abs_dim1                        = 4,
 cam_abs_dim2                        = 28,
 sf_sfclay_physics                   = 1,     1,     1,
 sf_surface_physics                  = 1,     1,     1,
 bl_pbl_physics                      = 1,     1,     1,
 bldt                                = 0,     0,     0,
 cu_physics                          = 1,     1,     0,
 cudt                                = 5,     5,     5,
 isfflx                              = 1,
 ifsnow                              = 0,
 icloud                              = 1,
 surface_input_source                = 1,
 num_soil_layers                     = 5,
 ucmcall                             = 0,
 mp_zero_out                         = 0,
 maxiens                             = 1,
 maxens                              = 3,
 maxens2                             = 3,
 maxens3                             = 16,
 ensdim                              = 144,
 seaice_threshold                    = 271,
 sst_update                          = 1
 /

 &fdda
 grid_fdda                           = 1,     1,     1,
 gfdda_inname                        = "wrffdda_d<domain>",
 gfdda_end_h                         = 744,    24,    24,
 gfdda_interval_m                    = 360,   360,   360,
 fgdt                                = 0,     0,     0,
 if_no_pbl_nudging_uv                = 0,     0,     1,
 if_no_pbl_nudging_t                 = 0,     0,     1,
 if_no_pbl_nudging_q                 = 0,     0,     1,
 if_zfac_uv                          = 0,     0,     1,
  k_zfac_uv                          = 10,   10,     1,
 if_zfac_t                           = 0,     0,     1,
  k_zfac_t                           = 10,   10,     1,
 if_zfac_q                           = 0,     0,     1,
  k_zfac_q                           = 10,   10,     1,
 guv                                 = 0.0003,     0.0003,     0.0003,
 gt                                  = 0.0003,     0.0003,     0.0003,
 gq                                  = 0.0003,     0.0003,     0.0003,
 if_ramping                          = 1,
 dtramp_min                          = 60.0,  
 io_form_gfdda                       = 2,
 /

 &dynamics
 w_damping                           = 0,
 diff_opt                            = 1,
 km_opt                              = 4,
 diff_6th_opt                        = 0,
 diff_6th_factor                     = 0.12,
 damp_opt                            = 0,
 base_temp                           = 290.
 zdamp                               = 5000.,  5000.,  5000.,
 dampcoef                            = 0.01,   0.01,   0.01
 khdif                               = 0,      0,      0,
 kvdif                               = 0,      0,      0,
 non_hydrostatic                     = .true., .true., .true.,
 pd_moist                            = .false., .false., .false.,
 pd_scalar                           = .false., .false., .false.,
 /

 &bdy_control
 spec_bdy_width                      = 10,
 spec_zone                           = 1,
 relax_zone                          = 9,
 specified                           = .true., .false.,.false.,
 nested                              = .false., .true., .true.,
 /

 &grib2
 /

 &namelist_quilt
 nio_tasks_per_group = 0,
 nio_groups = 1,
 /
EOF
ln -fs $realdir/wrfinput_d01.199405 	./wrfinput_d01
ln -fs $realdir/wrfbdy_d01.199405 	./wrfbdy_d01
ln -fs $realdir/wrflowinp_d01.199405 ./wrflowinp_d01
ln -fs $realdir/wrffdda_d01.199405 	./wrffdda_d01
real.exe > & real.199405.prt
rm wrfinput_d01 wrfbdy_d01 wrflowinp_d01 wrffdda_d01 real.exe
mv real.199405.prt $defdir/run
cd $defdir
real.run.199406.csh
